@@ -1,11 +1,11 @@
-# Copyright (c) 2017 MetPy Developers.
+# Copyright (c) 2017,2019 MetPy Developers.
 # Distributed under the terms of the BSD 3-Clause License.
 # SPDX-License-Identifier: BSD-3-Clause
 """Contains calculation of various derived indices."""
 import numpy as np
 
 from .thermo import mixing_ratio, saturation_vapor_pressure
-from .tools import get_layer
+from .tools import _remove_nans, get_layer
 from .. import constants as mpconsts
 from ..package_tools import Exporter
 from ..units import atleast_1d, check_units, concatenate, units
@@ -48,6 +48,8 @@ def precipitable_water(dewpt, pressure, bottom=None, top=None):
     pressure = pressure[sort_inds]
     dewpt = dewpt[sort_inds]
 
+    pressure, dewpt = _remove_nans(pressure, dewpt)
+
     if top is None:
         top = np.nanmin(pressure.magnitude) * pressure.units
 
