@@ -11,7 +11,7 @@ import numpy as np
 import scipy.integrate as si
 import scipy.optimize as so
 
-from .tools import (_greater_or_close, _less_or_close, find_bounding_indices,
+from .tools import (_greater_or_close, _less_or_close, _remove_nans, find_bounding_indices,
                     find_intersections, first_derivative, get_layer)
 from .. import constants as mpconsts
 from ..cbook import broadcast_indices
@@ -310,7 +310,8 @@ def lcl(pressure, temperature, dewpt, max_iters=50, eps=1e-5):
     r"""Calculate the lifted condensation level (LCL) using from the starting point.
 
     The starting state for the parcel is defined by `temperature`, `dewpt`,
-    and `pressure`.
+    and `pressure`. If these are arrays, this function will return a LCL
+    for every index. This function does work with surface grids as a result.
 
     Parameters
     ----------
@@ -356,9 +357,13 @@ def lcl(pressure, temperature, dewpt, max_iters=50, eps=1e-5):
         return (p0 * (td / t) ** (1. / mpconsts.kappa)).m
 
     w = mixing_ratio(saturation_vapor_pressure(dewpt), pressure)
-    fp = so.fixed_point(_lcl_iter, pressure.m, args=(pressure.m, w, temperature),
-                        xtol=eps, maxiter=max_iters)
-    lcl_p = fp * pressure.units
+    lcl_p = so.fixed_point(_lcl_iter, pressure.m, args=(pressure.m, w, temperature),
+                           xtol=eps, maxiter=max_iters)
+
+    # np.isclose needed if surface is LCL due to precision error with np.log in dewpoint.
+    # Causes issues with parcel_profile_with_lcl if removed. Issue #1187
+    lcl_p = np.where(np.isclose(lcl_p, pressure), pressure, lcl_p) * pressure.units
+
     return lcl_p, dewpoint(vapor_pressure(lcl_p, w)).to(temperature.units)
 
 
@@ -370,7 +375,10 @@ def lfc(pressure, temperature, dewpt, parcel_temperature_profile=None, dewpt_sta
     r"""Calculate the level of free convection (LFC).
 
     This works by finding the first intersection of the ideal parcel path and
-    the measured parcel temperature.
+    the measured parcel temperature. If this intersection occurs below the LCL,
+    the LFC is determined to be the same as the LCL, based upon the conditions
+    set forth in [USAF1990]_, pg 4-14, where a parcel must be lifted dry adiabatically
+    to saturation before it can freely rise.
 
     Parameters
     ----------
@@ -402,6 +410,7 @@ def lfc(pressure, temperature, dewpt, parcel_temperature_profile=None, dewpt_sta
     parcel_profile
 
     """
+    pressure, temperature, dewpt = _remove_nans(pressure, temperature, dewpt)
     # Default to surface parcel if no profile or starting pressure level is given
     if parcel_temperature_profile is None:
         new_stuff = parcel_profile_with_lcl(pressure, temperature, dewpt)
@@ -414,7 +423,7 @@ def lfc(pressure, temperature, dewpt, parcel_temperature_profile=None, dewpt_sta
     # The parcel profile and data may have the same first data point.
     # If that is the case, ignore that point to get the real first
     # intersection for the LFC calculation. Use logarithmic interpolation.
-    if np.isclose(parcel_temperature_profile[0].m, temperature[0].m):
+    if np.isclose(parcel_temperature_profile[0].to(temperature.units).m, temperature[0].m):
         x, y = find_intersections(pressure[1:], parcel_temperature_profile[1:],
                                   temperature[1:], direction='increasing', log_x=True)
     else:
@@ -509,6 +518,7 @@ def el(pressure, temperature, dewpt, parcel_temperature_profile=None, which='top
     parcel_profile
 
     """
+    pressure, temperature, dewpt = _remove_nans(pressure, temperature, dewpt)
     # Default to surface parcel if no profile or starting pressure level is given
     if parcel_temperature_profile is None:
         new_stuff = parcel_profile_with_lcl(pressure, temperature, dewpt)
@@ -1425,6 +1435,8 @@ def cape_cin(pressure, temperature, dewpt, parcel_profile):
     lfc, el
 
     """
+    pressure, temperature, dewpt, parcel_profile = _remove_nans(pressure, temperature, dewpt,
+                                                                parcel_profile)
     # Calculate LFC limit of integration
     lfc_pressure, _ = lfc(pressure, temperature, dewpt,
                           parcel_temperature_profile=parcel_profile)
@@ -1467,6 +1479,9 @@ def cape_cin(pressure, temperature, dewpt, parcel_profile):
     cin = (mpconsts.Rd
            * (np.trapz(y_clipped, np.log(x_clipped)) * units.degK)).to(units('J/kg'))
 
+    # Set CIN to 0 if it's returned as a positive value (#1190)
+    if cin > 0 * units('J/kg'):
+        cin = 0 * units('J/kg')
     return cape, cin
 
 
@@ -1752,6 +1767,7 @@ def surface_based_cape_cin(pressure, temperature, dewpoint):
     cape_cin, parcel_profile
 
     """
+    pressure, temperature, dewpoint = _remove_nans(pressure, temperature, dewpoint)
     p, t, td, profile = parcel_profile_with_lcl(pressure, temperature, dewpoint)
     return cape_cin(p, t, td, profile)
 
@@ -1789,6 +1805,7 @@ def most_unstable_cape_cin(pressure, temperature, dewpoint, **kwargs):
     cape_cin, most_unstable_parcel, parcel_profile
 
     """
+    pressure, temperature, dewpoint = _remove_nans(pressure, temperature, dewpoint)
     _, _, _, parcel_idx = most_unstable_parcel(pressure, temperature, dewpoint, **kwargs)
     p, t, td, mu_profile = parcel_profile_with_lcl(pressure[parcel_idx:],
                                                    temperature[parcel_idx:],
