@@ -5,24 +5,23 @@
 
 from datetime import datetime
 
-import cartopy.crs as ccrs
-import cartopy.feature as cfeature
+try:
+    import cartopy.crs as ccrs
+    DEFAULT_LAT_LON = ccrs.PlateCarree()
+except ImportError:
+    DEFAULT_LAT_LON = None
 import matplotlib.pyplot as plt
+import numpy as np
 from traitlets import (Any, Bool, Float, HasTraits, Instance, Int, List, observe, Tuple,
                        Unicode, Union)
 
-from . import cartopy_utils, ctables
+from . import ctables
 from ..cbook import is_string_like
 from ..package_tools import Exporter
 from ..units import units
 
 exporter = Exporter(globals())
 
-_projections = {'lcc': ccrs.LambertConformal(central_latitude=40, central_longitude=-100,
-                                             standard_parallels=[30, 60]),
-                'ps': ccrs.NorthPolarStereo(central_longitude=-100),
-                'mer': ccrs.Mercator()}
-
 _areas = {
     '105': (-129.3, -22.37, 17.52, 53.78),
     'local': (-92., -64., 28.5, 48.5),
@@ -467,16 +466,51 @@ _areas = {
 }
 
 
+def lookup_projection(projection_code):
+    """Get a Cartopy projection based on a short abbreviation."""
+    import cartopy.crs as ccrs
+
+    projections = {'lcc': ccrs.LambertConformal(central_latitude=40, central_longitude=-100,
+                                                standard_parallels=[30, 60]),
+                   'ps': ccrs.NorthPolarStereo(central_longitude=-100),
+                   'mer': ccrs.Mercator()}
+    return projections[projection_code]
+
+
+def lookup_map_feature(feature_name):
+    """Get a Cartopy map feature based on a name."""
+    import cartopy.feature as cfeature
+    from . import cartopy_utils
+
+    name = feature_name.upper()
+    try:
+        feat = getattr(cfeature, name)
+        scaler = cfeature.AdaptiveScaler('110m', (('50m', 50), ('10m', 15)))
+    except AttributeError:
+        feat = getattr(cartopy_utils, name)
+        scaler = cfeature.AdaptiveScaler('20m', (('5m', 5), ('500k', 1)))
+    return feat.with_scale(scaler)
+
+
 class Panel(HasTraits):
     """Draw one or more plots."""
 
 
 @exporter.export
 class PanelContainer(HasTraits):
-    """Hold multiple panels of plots."""
+    """Collects panels and set complete figure related settings (e.g., figsize)."""
 
     size = Union([Tuple(Int(), Int()), Instance(type(None))], default_value=None)
+    size.__doc__ = """This trait takes a tuple of (width, height) to set the size of the
+    figure.
+
+    This trait defaults to None and will assume the default `matplotlib.pyplot.figure` size.
+    """
+
     panels = List(Instance(Panel))
+    panels.__doc__ = """A list of panels to plot on the figure.
+
+    This trait must contain at least one panel to plot on the figure."""
 
     @property
     def panel(self):
@@ -521,7 +555,11 @@ class PanelContainer(HasTraits):
                 panel.draw()
 
     def save(self, *args, **kwargs):
-        """Save the constructed graphic as an image file."""
+        """Save the constructed graphic as an image file.
+
+        This method takes a string for saved file name. Additionally, the same arguments and
+        keyword arguments that `matplotlib.pyplot.savefig` does.
+        """
         self.draw()
         self.figure.savefig(*args, **kwargs)
 
@@ -533,18 +571,76 @@ class PanelContainer(HasTraits):
 
 @exporter.export
 class MapPanel(Panel):
-    """Draw one or more plots on a map."""
+    """Set figure related elements for an individual panel.
+
+    Parameters that need to be set include collecting all plotting types
+    (e.g., contours, wind barbs, etc.) that are desired to be in a given panel.
+    Additionally, traits can be set to plot map related features (e.g., coastlines, borders),
+    projection, graphics area, and title.
+    """
 
     parent = Instance(PanelContainer)
+
     layout = Tuple(Int(), Int(), Int(), default_value=(1, 1, 1))
+    layout.__doc__ = """A tuple that contains the description (nrows, ncols, index) of the
+    panel position; default value is (1, 1, 1).
+
+    This trait is set to describe the panel position and the default is for a single panel. For
+    example, a four-panel plot will have two rows and two columns with the tuple setting for
+    the upper-left panel as (2, 2, 1), upper-right as (2, 2, 2), lower-left as (2, 2, 3), and
+    lower-right as (2, 2, 4). For more details see the documentation for
+    `matplotlib.figure.Figure.add_subplot`.
+    """
+
     plots = List(Any())
+    plots.__doc__ = """A list of handles that represent the plots (e.g., `ContourPlot`,
+    `FilledContourPlot`, `ImagePlot`) to put on a given panel.
+
+    This trait collects the different plots, including contours and images, that are intended
+    for a given panel.
+    """
+
     _need_redraw = Bool(default_value=True)
 
     area = Union([Unicode(), Tuple(Float(), Float(), Float(), Float())], allow_none=True,
                  default_value=None)
-    projection = Union([Unicode(), Instance(ccrs.Projection)], default_value='data')
-    layers = List(Union([Unicode(), Instance(cfeature.Feature)]), default_value=['coastline'])
+    area.__doc__ = """A tuple or string value that indicates the graphical area of the plot.
+
+    The tuple value coresponds to longitude/latitude box based on the projection of the map
+    with the format (west-most longitude, east-most longitude, south-most latitude,
+    north-most latitude). This tuple defines a box from the lower-left to the upper-right
+    corner.
+
+    This trait can also be set with a string value associated with the named geographic regions
+    within MetPy. The tuples associated with the names are based on a PlatteCarree projection.
+    For a CONUS region, the following strings can be used: 'us', 'spcus', 'ncus', and 'afus'.
+    For regional plots, US state postal codes can be used.
+    """
+
+    projection = Union([Unicode(), Instance('cartopy.crs.Projection')], default_value='data')
+    projection.__doc__ = """A string for a pre-defined projection or a Cartopy projection
+    object.
+
+    There are three pre-defined projections that can be called with a short name:
+    Lambert conformal conic ('lcc'), Mercator ('mer'), or polar-stereographic ('ps').
+    Additionally, this trait can be set to a Cartopy projection object.
+    """
+
+    layers = List(Union([Unicode(), Instance('cartopy.feature.Feature')]),
+                  default_value=['coastline'])
+    layers.__doc__ = """A string for a pre-defined feature layer or a Cartopy Feature object.
+
+    Like the projection, there are a couple of pre-defined feature layers that can be called
+    using a short name. The pre-defined layers are: 'coastline', 'states', 'borders', 'lakes',
+    'land', 'ocean', and 'rivers'. Additionally, this trait can be set using a Cartopy Feature
+    object.
+    """
+
     title = Unicode()
+    title.__doc__ = """A string to set a title for the figure.
+
+    This trait sets a user-defined title that will plot at the top center of the figure.
+    """
 
     @observe('plots')
     def _plots_changed(self, change):
@@ -569,9 +665,12 @@ class MapPanel(Panel):
         """
         if is_string_like(self.projection):
             if self.projection == 'data':
-                return self.plots[0].griddata.metpy.cartopy_crs
+                if isinstance(self.plots[0].griddata, tuple):
+                    return self.plots[0].griddata[0].metpy.cartopy_crs
+                else:
+                    return self.plots[0].griddata.metpy.cartopy_crs
             else:
-                return _projections[self.projection]
+                return lookup_projection(self.projection)
         else:
             return self.projection
 
@@ -584,13 +683,7 @@ class MapPanel(Panel):
         """
         for item in self.layers:
             if is_string_like(item):
-                item = item.upper()
-                try:
-                    scaler = cfeature.AdaptiveScaler('110m', (('50m', 50), ('10m', 15)))
-                    feat = getattr(cfeature, item).with_scale(scaler)
-                except AttributeError:
-                    scaler = cfeature.AdaptiveScaler('20m', (('5m', 5), ('500k', 1)))
-                    feat = getattr(cartopy_utils, item).with_scale(scaler)
+                feat = lookup_map_feature(item)
             else:
                 feat = item
 
@@ -654,28 +747,39 @@ class MapPanel(Panel):
                 # Otherwise, assume we have a tuple to use as the extent
                 else:
                     area = self.area
-                self.ax.set_extent(area, ccrs.PlateCarree())
+                self.ax.set_extent(area, DEFAULT_LAT_LON)
 
             # Use the set title or generate one.
-            title = self.title or ', '.join(plot.name for plot in self.plots)
+            title = self.title or ',\n'.join(plot.name for plot in self.plots)
             self.ax.set_title(title)
             self._need_redraw = False
 
 
 @exporter.export
-class Plot2D(HasTraits):
-    """Represent plots of 2D data."""
+class Plots2D(HasTraits):
+    """The highest level class related to plotting 2D data.
+
+    This class collects all common methods no matter whether plotting a scalar variable or
+    vector. Primary settings common to all types of 2D plots are time and level.
+    """
 
     parent = Instance(Panel)
     _need_redraw = Bool(default_value=True)
 
-    field = Unicode()
     level = Union([Int(allow_none=True, default_value=None), Instance(units.Quantity)])
-    time = Instance(datetime, allow_none=True)
+    level.__doc__ = """The level of the field to be plotted.
 
-    colormap = Unicode(allow_none=True, default_value=None)
-    image_range = Union([Tuple(Int(allow_none=True), Int(allow_none=True)),
-                         Instance(plt.Normalize)], default_value=(None, None))
+    This is a value with units to choose the desired plot level. For example, selecting the
+    850-hPa level, set this parameter to ``850 * units.hPa``
+    """
+
+    time = Instance(datetime, allow_none=True)
+    time.__doc__ = """Set the valid time to be plotted as a datetime object.
+
+    If a forecast hour is to be plotted the time should be set to the valid future time, which
+    can be done using the `~datetime.datetime` and `~datetime.timedelta` objects
+    from the Python standard library.
+    """
 
     @property
     def _cmap_obj(self):
@@ -705,7 +809,10 @@ class Plot2D(HasTraits):
 
         """
         if getattr(self, 'handle', None) is not None:
-            self.clear_handle()
+            if getattr(self.handle, 'collections', None) is not None:
+                self.clear_collections()
+            else:
+                self.clear_handle()
             self._need_redraw = True
 
     def clear_handle(self):
@@ -713,12 +820,18 @@ class Plot2D(HasTraits):
         self.handle.remove()
         self.handle = None
 
+    def clear_collections(self):
+        """Clear the handle collections to the plot instance."""
+        for col in self.handle.collections:
+            col.remove()
+        self.handle = None
+
     @observe('parent')
     def _parent_changed(self, _):
         """Handle setting the parent object for the plot."""
         self.clear()
 
-    @observe('field', 'level', 'time')
+    @observe('level', 'time')
     def _update_data(self, _=None):
         """Handle updating the internal cache of data.
 
@@ -732,7 +845,7 @@ class Plot2D(HasTraits):
     # notification never happens
     @property
     def data(self):
-        """Access the current data subset."""
+        """Xarray dataset that contains the field to be plotted."""
         return self._data
 
     @data.setter
@@ -740,6 +853,49 @@ class Plot2D(HasTraits):
         self._data = val
         self._update_data()
 
+    @property
+    def name(self):
+        """Generate a name for the plot."""
+        if isinstance(self.field, tuple):
+            ret = ''
+            ret += ' and '.join(f for f in self.field)
+        else:
+            ret = self.field
+        if self.level is not None:
+            ret += '@{:d}'.format(self.level)
+        return ret
+
+
+@exporter.export
+class PlotScalar(Plots2D):
+    """Defines the common elements of 2D scalar plots for single scalar value fields.
+
+    Most of the other traits here are for one or more of the specific plots. Currently this
+    allows too many options for `ContourPlot` since it does not user image_range, for
+    example. Similar issues for `ImagePlot` and `FilledContourPlot`.
+    """
+
+    field = Unicode()
+    field.__doc__ = """Name of the field to be plotted.
+
+    This is the name of the variable from the dataset that is to be plotted. An example,
+    from a model grid file that uses the THREDDS convention for naming would be
+    `Geopotential_height_isobaric` or `Temperature_isobaric`. For GOES-16/17 satellite data it
+    might be `Sectorized_CMI`. To check for the variables available within a dataset, list the
+    variables with the following command assuming the dataset was read using xarray as `ds`,
+    `list(ds)`
+    """
+
+    @observe('field')
+    def _update_data(self, _=None):
+        """Handle updating the internal cache of data.
+
+        Responds to changes in various subsetting parameters.
+
+        """
+        self._griddata = None
+        self.clear()
+
     @property
     def griddata(self):
         """Return the internal cached data."""
@@ -747,6 +903,7 @@ class Plot2D(HasTraits):
 
             if self.field:
                 data = self.data.metpy.parse_cf(self.field)
+
             elif not hasattr(self.data.metpy, 'x'):
                 # Handles the case where we have a dataset but no specified field
                 raise ValueError('field attribute has not been set.')
@@ -774,33 +931,72 @@ class Plot2D(HasTraits):
         y = self.griddata.metpy.y
 
         if 'degree' in x.units:
-            import numpy as np
-            x, y, _ = self.griddata.metpy.cartopy_crs.transform_points(ccrs.PlateCarree(),
+            x, y, _ = self.griddata.metpy.cartopy_crs.transform_points(DEFAULT_LAT_LON,
                                                                        *np.meshgrid(x, y)).T
             x = x[:, 0] % 360
             y = y[0, :]
 
         return x, y, self.griddata
 
-    @property
-    def name(self):
-        """Generate a name for the plot."""
-        ret = self.field
-        if self.level is not None:
-            ret += '@{:d}'.format(self.level)
-        return ret
-
     def draw(self):
         """Draw the plot."""
         if self._need_redraw:
             if getattr(self, 'handle', None) is None:
                 self._build()
+            if getattr(self, 'colorbar', None) is not None:
+                self.parent.ax.figure.colorbar(
+                    self.handle, orientation=self.colorbar, pad=0, aspect=50)
             self._need_redraw = False
 
 
+class ContourTraits(HasTraits):
+    """Represents common contour traits."""
+
+    contours = Union([List(Float()), Int()], default_value=25)
+    contours.__doc__ = """A list of values to contour or an integer number of contour levels.
+
+    This parameter sets contour or colorfill values for a plot. Values can be entered either
+    as a list of values or as an integer with the number of contours to be plotted (as per
+    matplotlib documentation). A list can be generated by using square brackets or creating
+    a numpy 1D array and converting it to a list with the `~numpy.ndarray.tolist` method.
+    """
+
+    clabels = Bool(default_value=False)
+    clabels.__doc__ = """A boolean (True/False) on whether to plot contour labels.
+
+    To plot contour labels set this trait to ``True``, the default value is ``False``.
+    """
+
+
+class ColorfillTraits(HasTraits):
+    """Represent common colorfill traits."""
+
+    colormap = Unicode(allow_none=True, default_value=None)
+    colormap.__doc__ = """The string name for a Matplolib or MetPy colormap.
+
+    For example, the Blue-Purple colormap from Matplotlib can be accessed using 'BuPu'.
+    """
+
+    image_range = Union([Tuple(Int(allow_none=True), Int(allow_none=True)),
+                         Instance(plt.Normalize)], default_value=(None, None))
+    image_range.__doc__ = """A tuple of min and max values that represent the range of values
+    to color the rasterized image.
+
+    The min and max values entered as a tuple will be converted to a
+    `matplotlib.colors.Normalize` instance for plotting.
+    """
+
+    colorbar = Unicode(default_value=None, allow_none=True)
+    colorbar.__doc__ = """A boolean (True/False) on whether to add a colorbar to the plot.
+
+    To add a colorbar associated with the plot data set the trait to ``True``, the default
+    values is ``False``.
+    """
+
+
 @exporter.export
-class ImagePlot(Plot2D):
-    """Represent an image plot."""
+class ImagePlot(PlotScalar, ColorfillTraits):
+    """Make raster image using `~matplotlib.pyplot.imshow` for satellite or colored image."""
 
     @observe('colormap', 'image_range')
     def _set_need_redraw(self, _):
@@ -810,6 +1006,13 @@ class ImagePlot(Plot2D):
             self.handle.set_norm(self._norm_obj)
             self._need_redraw = True
 
+    @observe('colorbar')
+    def _set_need_rebuild(self, _):
+        """Handle changes to attributes that need to regenerate everything."""
+        # Because matplotlib doesn't let you just change these properties, we need
+        # to trigger a clear and re-call of contour()
+        self.clear()
+
     @property
     def plotdata(self):
         """Return the data for plotting.
@@ -841,29 +1044,222 @@ class ImagePlot(Plot2D):
 
 
 @exporter.export
-class ContourPlot(Plot2D):
-    """Represent a contour plot."""
+class ContourPlot(PlotScalar, ContourTraits):
+    """Make contour plots by defining specific traits."""
 
     linecolor = Unicode('black')
-    linewidth = Int(2)
-    contours = Union([List(Float()), Int()], default_value=25)
+    linecolor.__doc__ = """A string value to set the color of plotted contours; default is
+    black.
 
-    @observe('contours', 'linecolor', 'linewidth')
+    This trait can be set to any Matplotlib color
+    (https://matplotlib.org/3.1.0/gallery/color/named_colors.html)
+    """
+
+    linewidth = Int(2)
+    linewidth.__doc__ = """An integer value to set the width of plotted contours; default value
+    is 2.
+
+    This trait changes the thickness of contour lines with a higher value plotting a thicker
+    line.
+    """
+
+    linestyle = Unicode('solid', allow_none=True)
+    linestyle.__doc__ = """A string value to set the linestyle (e.g., dashed); default is
+    solid.
+
+    The valid string values are those of Matplotlib which are solid, dashed, dotted, and
+    dashdot.
+    """
+
+    @observe('contours', 'linecolor', 'linewidth', 'linestyle', 'clabels')
     def _set_need_rebuild(self, _):
         """Handle changes to attributes that need to regenerate everything."""
         # Because matplotlib doesn't let you just change these properties, we need
         # to trigger a clear and re-call of contour()
         self.clear()
 
-    def clear_handle(self):
-        """Clear the handle to the plot instance."""
-        for col in self.handle.collections:
-            col.remove()
-        self.handle = None
-
     def _build(self):
         """Build the plot by calling any plotting methods as necessary."""
         x, y, imdata = self.plotdata
         self.handle = self.parent.ax.contour(x, y, imdata, self.contours,
                                              colors=self.linecolor, linewidths=self.linewidth,
+                                             linestyles=self.linestyle,
                                              transform=imdata.metpy.cartopy_crs)
+        if self.clabels:
+            self.handle.clabel(inline=1, fmt='%.0f', inline_spacing=2,
+                               use_clabeltext=True)
+
+
+@exporter.export
+class FilledContourPlot(PlotScalar, ColorfillTraits, ContourTraits):
+    """Make color-filled contours plots by defining appropriate traits."""
+
+    @observe('contours', 'colorbar', 'colormap')
+    def _set_need_rebuild(self, _):
+        """Handle changes to attributes that need to regenerate everything."""
+        # Because matplotlib doesn't let you just change these properties, we need
+        # to trigger a clear and re-call of contour()
+        self.clear()
+
+    def _build(self):
+        """Build the plot by calling any plotting methods as necessary."""
+        x, y, imdata = self.plotdata
+        self.handle = self.parent.ax.contourf(x, y, imdata, self.contours,
+                                              cmap=self._cmap_obj, norm=self._norm_obj,
+                                              transform=imdata.metpy.cartopy_crs)
+
+
+@exporter.export
+class PlotVector(Plots2D):
+    """Defines common elements for 2D vector plots.
+
+    This class collects common elements including the field trait, which is a tuple argument
+    accepting two strings, for plotting 2D vector fields.
+    """
+
+    field = Tuple(Unicode(), Unicode())
+    field.__doc__ = """A tuple containing the two components of the vector field from the
+    dataset in the form (east-west component, north-south component).
+
+    For a wind barb plot each component of the wind must be specified and should be of the form
+    (u-wind, v-wind).
+    """
+
+    pivot = Unicode('middle')
+    pivot.__doc__ = """A string setting the pivot point of the vector. Default value is
+    'middle'.
+
+    This trait takes the values of the keyword argument from `matplotlin.pyplot.barbs`:
+    'tip' or 'middle'.
+    """
+
+    skip = Tuple(Int(), Int(), default_value=(1, 1))
+    skip.__doc__ = """A tuple of integers to indicate the number of grid points to skip between
+    plotting vectors. Default is (1, 1).
+
+    This trait is to be used to reduce the number of vectors plotted in the (east-west,
+    north-south) components. The two values can be set to the same or different integer values
+    depending on what is desired.
+    """
+
+    earth_relative = Bool(default_value=True)
+    earth_relative.__doc__ = """A boolean value to indicate whether the vector to be plotted
+    is earth- or grid-relative. Default value is `True`, indicating that vectors are
+    earth-relative.
+
+    Common gridded meteorological datasets including GFS and NARR output contain wind
+    components that are earth-relative. The primary expection is NAM output with wind
+    components that are grid-relative. For any grid-relative vectors set this trait to `False`.
+    """
+
+    color = Unicode(default_value='black')
+    color.__doc__ = """A string value that controls the color of the vectors. Default value is
+    black.
+
+    This trait can be set to any named color from
+    `Matplotlibs Colors <https://matplotlib.org/3.1.0/gallery/color/named_colors.html>`
+    """
+
+    @observe('field')
+    def _update_data(self, _=None):
+        """Handle updating the internal cache of data.
+
+        Responds to changes in various subsetting parameters.
+
+        """
+        self._griddata_u = None
+        self._griddata_v = None
+        self.clear()
+
+    @property
+    def griddata(self):
+        """Return the internal cached data."""
+        if getattr(self, '_griddata_u', None) is None:
+
+            if self.field[0]:
+                u = self.data.metpy.parse_cf(self.field[0])
+                v = self.data.metpy.parse_cf(self.field[1])
+            else:
+                raise ValueError('field attribute not set correctly')
+
+            subset = {'method': 'nearest'}
+            if self.level is not None:
+                subset[u.metpy.vertical.name] = self.level
+
+            if self.time is not None:
+                subset[u.metpy.time.name] = self.time
+            self._griddata_u = u.metpy.sel(**subset).squeeze()
+            self._griddata_v = v.metpy.sel(**subset).squeeze()
+
+        return (self._griddata_u, self._griddata_v)
+
+    @property
+    def plotdata(self):
+        """Return the data for plotting.
+
+        The data array, x coordinates, and y coordinates.
+
+        """
+        x = self.griddata[0].metpy.x
+        y = self.griddata[0].metpy.y
+
+        if self.earth_relative:
+            x, y, _ = DEFAULT_LAT_LON.transform_points(self.griddata[0].metpy.cartopy_crs,
+                                                       *np.meshgrid(x, y)).T
+            x = x.T
+            y = y.T
+        else:
+            if 'degree' in x.units:
+                x, y, _ = self.griddata[0].metpy.cartopy_crs.transform_points(
+                    DEFAULT_LAT_LON, *np.meshgrid(x, y)).T
+                x = x.T % 360
+                y = y.T
+
+        if x.ndim == 1:
+            xx, yy = np.meshgrid(x, y)
+        else:
+            xx, yy = x, y
+
+        return xx, yy, self.griddata[0], self.griddata[1]
+
+    def draw(self):
+        """Draw the plot."""
+        if self._need_redraw:
+            if getattr(self, 'handle', None) is None:
+                self._build()
+            self._need_redraw = False
+
+
+@exporter.export
+class BarbPlot(PlotVector):
+    """Make plots of wind barbs on a map with traits to refine the look of plotted elements."""
+
+    barblength = Float(default_value=7)
+    barblength.__doc__ = """A float value that changes the length of the wind barbs. Default
+    value is 7.
+
+    This trait corresponds to the keyword length in `matplotlib.pyplot.barbs`.
+    """
+
+    @observe('barblength', 'pivot', 'skip', 'earth_relative', 'color')
+    def _set_need_rebuild(self, _):
+        """Handle changes to attributes that need to regenerate everything."""
+        # Because matplotlib doesn't let you just change these properties, we need
+        # to trigger a clear and re-call of contour()
+        self.clear()
+
+    def _build(self):
+        """Build the plot by calling needed plotting methods as necessary."""
+        x, y, u, v = self.plotdata
+        if self.earth_relative:
+            transform = DEFAULT_LAT_LON
+        else:
+            transform = u.metpy.cartopy_crs
+
+        wind_slice = (slice(None, None, self.skip[0]), slice(None, None, self.skip[1]))
+
+        self.handle = self.parent.ax.barbs(
+            x[wind_slice], y[wind_slice],
+            u.values[wind_slice], v.values[wind_slice],
+            color=self.color, pivot=self.pivot, length=self.barblength,
+            transform=transform)
