@@ -1,4 +1,4 @@
-# Copyright (c) 2014,2015,2016,2017 MetPy Developers.
+# Copyright (c) 2014,2015,2016,2017,2019 MetPy Developers.
 # Distributed under the terms of the BSD 3-Clause License.
 # SPDX-License-Identifier: BSD-3-Clause
 """Make Skew-T Log-P based plots.
@@ -7,6 +7,7 @@ Contain tools for making Skew-T Log-P plots, including the base plotting class,
 `SkewT`, as well as a class for making a `Hodograph`.
 """
 
+import warnings
 try:
     from contextlib import ExitStack
 except ImportError:
@@ -27,6 +28,7 @@ import numpy as np
 from ._util import colored_line
 from ..calc import dewpoint, dry_lapse, moist_lapse, vapor_pressure
 from ..calc.tools import _delete_masked_points
+from ..deprecation import metpyDeprecation
 from ..interpolate import interpolate_1d
 from ..package_tools import Exporter
 from ..units import concatenate, units
@@ -34,6 +36,47 @@ from ..units import concatenate, units
 exporter = Exporter(globals())
 
 
+class SkewTTransform(transforms.Affine2D):
+    """Perform Skew transform for Skew-T plotting.
+
+    This works in pixel space, so is designed to be applied after the normal plotting
+    transformations.
+    """
+
+    def __init__(self, bbox, rot):
+        """Initialize skew transform.
+
+        This needs a reference to the parent bounding box to do the appropriate math and
+        to register it as a child so that the transform is invalidated and regenerated if
+        the bounding box changes.
+        """
+        super(transforms.Affine2D, self).__init__()
+        self._bbox = bbox
+        self.set_children(bbox)
+        self.invalidate()
+
+        # We're not trying to support changing the rotation, so go ahead and convert to
+        # the right factor for skewing here and just save that.
+        self._rot_factor = np.tan(np.deg2rad(rot))
+
+    def get_matrix(self):
+        """Return transformation matrix."""
+        if self._invalid:
+            # The following matrix is equivalent to the following:
+            # x0, y0 = self._bbox.xmin, self._bbox.ymin
+            # self.translate(-x0, -y0).skew_deg(self._rot, 0).translate(x0, y0)
+            # Setting it this way is just more efficient.
+            self._mtx = np.array([[1.0, self._rot_factor, -self._rot_factor * self._bbox.ymin],
+                                  [0.0, 1.0, 0.0],
+                                  [0.0, 0.0, 1.0]])
+
+            # Need to clear both the invalid flag *and* reset the inverse, which is cached
+            # by the parent class.
+            self._invalid = 0
+            self._inverted = None
+        return self._mtx
+
+
 class SkewXTick(maxis.XTick):
     r"""Make x-axis ticks for Skew-T plots.
 
@@ -54,36 +97,50 @@ class SkewXTick(maxis.XTick):
             for artist in [self.gridline, self.tick1line, self.tick2line,
                            self.label1, self.label2]:
                 stack.callback(artist.set_visible, artist.get_visible())
-            needs_lower = transforms.interval_contains(
-                self.axes.lower_xlim, self.get_loc())
-            needs_upper = transforms.interval_contains(
-                self.axes.upper_xlim, self.get_loc())
-            self.tick1line.set_visible(
-                self.tick1line.get_visible() and needs_lower)
-            self.label1.set_visible(
-                self.label1.get_visible() and needs_lower)
-            self.tick2line.set_visible(
-                self.tick2line.get_visible() and needs_upper)
-            self.label2.set_visible(
-                self.label2.get_visible() and needs_upper)
+
+            self.tick1line.set_visible(self.tick1line.get_visible() and self.lower_in_bounds)
+            self.label1.set_visible(self.label1.get_visible() and self.lower_in_bounds)
+            self.tick2line.set_visible(self.tick2line.get_visible() and self.upper_in_bounds)
+            self.label2.set_visible(self.label2.get_visible() and self.upper_in_bounds)
+            self.gridline.set_visible(self.gridline.get_visible() and self.grid_in_bounds)
             super(SkewXTick, self).draw(renderer)
 
-    def get_view_interval(self):
-        """Get the view interval."""
-        return self.axes.xaxis.get_view_interval()
+    @property
+    def lower_in_bounds(self):
+        """Whether the lower part of the tick is in bounds."""
+        return transforms.interval_contains(self.axes.lower_xlim, self.get_loc())
+
+    @property
+    def upper_in_bounds(self):
+        """Whether the upper part of the tick is in bounds."""
+        return transforms.interval_contains(self.axes.upper_xlim, self.get_loc())
+
+    @property
+    def grid_in_bounds(self):
+        """Whether any of the tick grid line is in bounds."""
+        return transforms.interval_contains(self.axes.xaxis.get_view_interval(),
+                                            self.get_loc())
 
 
 class SkewXAxis(maxis.XAxis):
     r"""Make an x-axis that works properly for Skew-T plots.
 
     This class exists to force the use of our custom :class:`SkewXTick` as well
-    as provide a custom value for interview that combines the extents of the
+    as provide a custom value for interval that combines the extents of the
     upper and lower x-limits from the axes.
     """
 
     def _get_tick(self, major):
         return SkewXTick(self.axes, None, '', major=major)
 
+    # Needed to properly handle tight bbox
+    def _get_tick_bboxes(self, ticks, renderer):
+        """Return lists of bboxes for ticks' label1's and label2's."""
+        return ([tick.label1.get_window_extent(renderer)
+                 for tick in ticks if tick.label1.get_visible() and tick.lower_in_bounds],
+                [tick.label2.get_window_extent(renderer)
+                 for tick in ticks if tick.label2.get_visible() and tick.upper_in_bounds])
+
     def get_view_interval(self):
         """Get the view interval."""
         return self.axes.upper_xlim[0], self.axes.lower_xlim[1]
@@ -164,24 +221,15 @@ class SkewXAxes(Axes):
         # Get the standard transform setup from the Axes base class
         super(Axes, self)._set_lim_and_transforms()
 
-        # Need to put the skew in the middle, after the scale and limits,
-        # but before the transAxes. This way, the skew is done in Axes
-        # coordinates thus performing the transform around the proper origin
-        # We keep the pre-transAxes transform around for other users, like the
-        # spines for finding bounds
-        self.transDataToAxes = (self.transScale
-                                + (self.transLimits
-                                   + transforms.Affine2D().skew_deg(self.rot, 0)))
+        # This transformation handles the skewing
+        skew_trans = SkewTTransform(self.bbox, self.rot)
 
         # Create the full transform from Data to Pixels
-        self.transData = self.transDataToAxes + self.transAxes
+        self.transData += skew_trans
 
         # Blended transforms like this need to have the skewing applied using
         # both axes, in axes coords like before.
-        self._xaxis_transform = (
-            transforms.blended_transform_factory(self.transScale + self.transLimits,
-                                                 transforms.IdentityTransform())
-            + transforms.Affine2D().skew_deg(self.rot, 0)) + self.transAxes
+        self._xaxis_transform += skew_trans
 
     @property
     def lower_xlim(self):
@@ -191,11 +239,11 @@ class SkewXAxes(Axes):
     @property
     def upper_xlim(self):
         """Get the data limits for the x-axis along the top of the axes."""
-        return self.transDataToAxes.inverted().transform([[0., 1.], [1., 1.]])[:, 0]
+        return self.transData.inverted().transform([[self.bbox.xmin, self.bbox.ymax],
+                                                    self.bbox.max])[:, 0]
 
 
-# Now register the projection with matplotlib so the user can select
-# it.
+# Now register the projection with matplotlib so the user can select it.
 register_projection(SkewXAxes)
 
 
@@ -216,7 +264,7 @@ class SkewT(object):
 
     """
 
-    def __init__(self, fig=None, rotation=30, subplot=None, rect=None):
+    def __init__(self, fig=None, rotation=30, subplot=None, rect=None, aspect=80.5):
         r"""Create SkewT - logP plots.
 
         Parameters
@@ -237,6 +285,10 @@ class SkewT(object):
         rect : tuple[float, float, float, float], optional
             Rectangle (left, bottom, width, height) in which to place the axes. This
             allows the user to place the axes at an arbitrary point on the figure.
+        aspect : float, int, or 'auto', optional
+            Aspect ratio (i.e. ratio of y-scale to x-scale) to maintain in the plot.
+            Defaults to 80.5. Passing the string ``'auto'`` tells matplotlib to handle
+            the aspect ratio automatically (this is not recommended for SkewT).
 
         """
         if fig is None:
@@ -262,14 +314,39 @@ class SkewT(object):
                 subplot = (1, 1, 1)
 
             self.ax = fig.add_subplot(*subplot, projection='skewx', rotation=rotation)
+
+        # Set the yaxis as inverted with log scaling
+        self.ax.set_yscale('log')
+
+        # Override default ticking for log scaling
+        self.ax.yaxis.set_major_formatter(ScalarFormatter())
+        self.ax.yaxis.set_major_locator(MultipleLocator(100))
+        self.ax.yaxis.set_minor_formatter(NullFormatter())
+
+        # Needed to make sure matplotlib doesn't freak out and create a bunch of ticks
+        # Also takes care of inverting the y-axis
+        self.ax.set_ylim(1050, 100)
+        self.ax.yaxis.set_units(units.hPa)
+
+        # Try to make sane default temperature plotting ticks
+        self.ax.xaxis.set_major_locator(MultipleLocator(10))
+        self.ax.xaxis.set_units(units.degC)
+        self.ax.set_xlim(-40, 50)
         self.ax.grid(True)
 
+        self.mixing_lines = None
+        self.dry_adiabats = None
+        self.moist_adiabats = None
+
+        # Maintain a reasonable ratio of data limits. Only works on Matplotlib >= 3.2
+        if matplotlib.__version__[:3] > '3.1':
+            self.ax.set_aspect(aspect, adjustable='box')
+
     def plot(self, p, t, *args, **kwargs):
         r"""Plot data.
 
         Simple wrapper around plot so that pressure is the first (independent)
-        input. This is essentially a wrapper around `semilogy`. It also
-        sets some appropriate ticking and plot ranges.
+        input. This is essentially a wrapper around `plot`.
 
         Parameters
         ----------
@@ -278,9 +355,9 @@ class SkewT(object):
         t : array_like
             temperature values, can also be used for things like dew point
         args
-            Other positional arguments to pass to :func:`~matplotlib.pyplot.semilogy`
+            Other positional arguments to pass to :func:`~matplotlib.pyplot.plot`
         kwargs
-            Other keyword arguments to pass to :func:`~matplotlib.pyplot.semilogy`
+            Other keyword arguments to pass to :func:`~matplotlib.pyplot.plot`
 
         Returns
         -------
@@ -289,24 +366,12 @@ class SkewT(object):
 
         See Also
         --------
-        :func:`matplotlib.pyplot.semilogy`
+        :func:`matplotlib.pyplot.plot`
 
         """
         # Skew-T logP plotting
         t, p = _delete_masked_points(t, p)
-        lines = self.ax.semilogy(t, p, *args, **kwargs)
-
-        # Disables the log-formatting that comes with semilogy
-        self.ax.yaxis.set_major_formatter(ScalarFormatter())
-        self.ax.yaxis.set_major_locator(MultipleLocator(100))
-        self.ax.yaxis.set_minor_formatter(NullFormatter())
-        if not self.ax.yaxis_inverted():
-            self.ax.invert_yaxis()
-
-        # Try to make sane default temperature plotting
-        self.ax.xaxis.set_major_locator(MultipleLocator(10))
-
-        return lines
+        return self.ax.plot(t, p, *args, **kwargs)
 
     def plot_barbs(self, p, u, v, c=None, xloc=1.0, x_clip_radius=0.1,
                    y_clip_radius=0.08, **kwargs):
@@ -368,11 +433,11 @@ class SkewT(object):
         if c is not None:
             b = self.ax.barbs(x, p, u, v, c,
                               transform=self.ax.get_yaxis_transform(which='tick2'),
-                              clip_on=True, **kwargs)
+                              clip_on=True, zorder=2, **kwargs)
         else:
             b = self.ax.barbs(x, p, u, v,
                               transform=self.ax.get_yaxis_transform(which='tick2'),
-                              clip_on=True, **kwargs)
+                              clip_on=True, zorder=2, **kwargs)
 
         # Override the default clip box, which is the axes rectangle, so we can have
         # barbs that extend outside.
@@ -413,6 +478,10 @@ class SkewT(object):
         :class:`matplotlib.collections.LineCollection`
 
         """
+        # Remove old lines
+        if self.dry_adiabats:
+            self.dry_adiabats.remove()
+
         # Determine set of starting temps if necessary
         if t0 is None:
             xmin, xmax = self.ax.get_xlim()
@@ -424,13 +493,14 @@ class SkewT(object):
 
         # Assemble into data for plotting
         t = dry_lapse(p, t0[:, np.newaxis], 1000. * units.mbar).to(units.degC)
-        linedata = [np.vstack((ti, p)).T for ti in t]
+        linedata = [np.vstack((ti.m, p.m)).T for ti in t]
 
         # Add to plot
         kwargs.setdefault('colors', 'r')
         kwargs.setdefault('linestyles', 'dashed')
         kwargs.setdefault('alpha', 0.5)
-        return self.ax.add_collection(LineCollection(linedata, **kwargs))
+        self.dry_adiabats = self.ax.add_collection(LineCollection(linedata, **kwargs))
+        return self.dry_adiabats
 
     def plot_moist_adiabats(self, t0=None, p=None, **kwargs):
         r"""Plot moist adiabats.
@@ -465,6 +535,10 @@ class SkewT(object):
         :class:`matplotlib.collections.LineCollection`
 
         """
+        # Remove old lines
+        if self.moist_adiabats:
+            self.moist_adiabats.remove()
+
         # Determine set of starting temps if necessary
         if t0 is None:
             xmin, xmax = self.ax.get_xlim()
@@ -477,13 +551,14 @@ class SkewT(object):
 
         # Assemble into data for plotting
         t = moist_lapse(p, t0[:, np.newaxis], 1000. * units.mbar).to(units.degC)
-        linedata = [np.vstack((ti, p)).T for ti in t]
+        linedata = [np.vstack((ti.m, p.m)).T for ti in t]
 
         # Add to plot
         kwargs.setdefault('colors', 'b')
         kwargs.setdefault('linestyles', 'dashed')
         kwargs.setdefault('alpha', 0.5)
-        return self.ax.add_collection(LineCollection(linedata, **kwargs))
+        self.moist_adiabats = self.ax.add_collection(LineCollection(linedata, **kwargs))
+        return self.moist_adiabats
 
     def plot_mixing_lines(self, w=None, p=None, **kwargs):
         r"""Plot lines of constant mixing ratio.
@@ -514,6 +589,10 @@ class SkewT(object):
         :class:`matplotlib.collections.LineCollection`
 
         """
+        # Remove old lines
+        if self.mixing_lines:
+            self.mixing_lines.remove()
+
         # Default mixing level values if necessary
         if w is None:
             w = np.array([0.0004, 0.001, 0.002, 0.004, 0.007, 0.01,
@@ -525,13 +604,14 @@ class SkewT(object):
 
         # Assemble data for plotting
         td = dewpoint(vapor_pressure(p, w))
-        linedata = [np.vstack((t, p)).T for t in td]
+        linedata = [np.vstack((t.m, p.m)).T for t in td]
 
         # Add to plot
         kwargs.setdefault('colors', 'g')
         kwargs.setdefault('linestyles', 'dashed')
         kwargs.setdefault('alpha', 0.8)
-        return self.ax.add_collection(LineCollection(linedata, **kwargs))
+        self.mixing_lines = self.ax.add_collection(LineCollection(linedata, **kwargs))
+        return self.mixing_lines
 
     def shade_area(self, y, x1, x2=0, which='both', **kwargs):
         r"""Shade area between two curves.
@@ -582,8 +662,7 @@ class SkewT(object):
             arrs = arrs + (fill_args['where'],)
             fill_args.pop('where', None)
 
-        if matplotlib.__version__ >= '2.1':
-            fill_args['interpolate'] = True
+        fill_args['interpolate'] = True
 
         arrs = _delete_masked_points(*arrs)
 
@@ -791,20 +870,17 @@ class Hodograph(object):
         return self.ax.quiver(center_position, center_position,
                               u, v, **quiver_args)
 
-    def plot_colormapped(self, u, v, c, bounds=None, colors=None, **kwargs):
+    def plot_colormapped(self, u, v, c, intervals=None, colors=None, **kwargs):
         r"""Plot u, v data, with line colored based on a third set of data.
 
         Plots the wind data on the hodograph, but with a colormapped line. Takes a third
-        variable besides the winds and either a colormap to color it with or a series of
-        bounds and colors to create a colormap and norm to control colormapping.
-        The bounds must always be in increasing order. For using custom bounds with
-        height data, the function will automatically interpolate to the actual bounds from the
-        height and wind data, as well as convert the input bounds from
-        height AGL to height above MSL to work with the provided heights.
-
-        Simple wrapper around plot so that pressure is the first (independent)
-        input. This is essentially a wrapper around `semilogy`. It also
-        sets some appropriate ticking and plot ranges.
+        variable besides the winds (e.g. heights or pressure levels) and either a colormap to
+        color it with or a series of contour intervals and colors to create a colormap and
+        norm to control colormapping. The intervals must always be in increasing
+        order. For using custom contour intervals with height data, the function will
+        automatically interpolate to the contour intervals from the height and wind data,
+        as well as convert the input contour intervals from height AGL to MSL to work with the
+        provided heights.
 
         Parameters
         ----------
@@ -813,9 +889,9 @@ class Hodograph(object):
         v : array_like
             v-component of wind
         c : array_like
-            data to use for colormapping
-        bounds: array-like, optional
-            Array of bounds for c to use in coloring the hodograph.
+            data to use for colormapping (e.g. heights, pressure, wind speed)
+        intervals: array-like, optional
+            Array of intervals for c to use in coloring the hodograph.
         colors: list, optional
             Array of strings representing colors for the hodograph segments.
         kwargs
@@ -830,19 +906,30 @@ class Hodograph(object):
         --------
         :meth:`Hodograph.plot`
 
+        Notes
+        -----
+        `plot_colormapped` previously accepted `bounds` as a parameter for coordinate values.
+        This has been deprecated in 0.11 in favor of `intervals`.
+
         """
+        bounds = kwargs.pop('bounds', None)
+        if bounds is not None:
+            intervals = bounds
+            warnings.warn('The use of "bounds" as a parameter has been deprecated in '
+                          'favor of "intervals",', metpyDeprecation)
+
         u, v, c = _delete_masked_points(u, v, c)
 
         # Plotting a color segmented hodograph
         if colors:
             cmap = mcolors.ListedColormap(colors)
-            # If we are segmenting by height (a length), interpolate the bounds
-            if bounds.dimensionality == {'[length]': 1.0}:
+            # If we are segmenting by height (a length), interpolate the contour intervals
+            if intervals.dimensionality == {'[length]': 1.0}:
 
-                # Find any bounds not in the data and interpolate them
-                interpolation_heights = [bound.m for bound in bounds if bound not in c]
-                interpolation_heights = np.array(interpolation_heights) * bounds.units
-                interpolation_heights = (np.sort(interpolation_heights)
+                # Find any intervals not in the data and interpolate them
+                interpolation_heights = [bound.m for bound in intervals if bound not in c]
+                interpolation_heights = np.array(interpolation_heights) * intervals.units
+                interpolation_heights = (np.sort(interpolation_heights.magnitude)
                                          * interpolation_heights.units)
                 (interpolated_heights, interpolated_u,
                  interpolated_v) = interpolate_1d(interpolation_heights, c, c, u, v)
@@ -859,12 +946,12 @@ class Hodograph(object):
                 # Unit conversion required for coloring of bounds/data in dissimilar units
                 # to work properly.
                 c = c.to_base_units()  # TODO: This shouldn't be required!
-                bounds = bounds.to_base_units()
+                intervals = intervals.to_base_units()
             # If segmenting by anything else, do not interpolate, just use the data
             else:
-                bounds = np.asarray(bounds) * bounds.units
+                intervals = np.asarray(intervals) * intervals.units
 
-            norm = mcolors.BoundaryNorm(bounds.magnitude, cmap.N)
+            norm = mcolors.BoundaryNorm(intervals.magnitude, cmap.N)
             cmap.set_over('none')
             cmap.set_under('none')
             kwargs['cmap'] = cmap
