@@ -15,6 +15,7 @@ from .wx_symbols import (current_weather, high_clouds, low_clouds, mid_clouds,
                          pressure_tendency, sky_cover, wx_symbol_font)
 from ..cbook import is_string_like
 from ..package_tools import Exporter
+from ..units import atleast_1d
 
 exporter = Exporter(globals())
 
@@ -59,8 +60,8 @@ class StationPlot(object):
 
         """
         self.ax = ax
-        self.x = x
-        self.y = y
+        self.x = atleast_1d(x)
+        self.y = atleast_1d(y)
         self.fontsize = fontsize
         self.spacing = fontsize if spacing is None else spacing
         self.transform = transform
